/* global wc_wooshpay_params */

jQuery( function( $ ) {
	'use strict';

	var wc_wooshpay_form = {
		init: function() {
			// checkout page
			if ( $( 'form.woocommerce-checkout' ).length ) {
				this.form = $( 'form.woocommerce-checkout' );
			}

			$( 'form.woocommerce-checkout' )
				.on(
					'checkout_place_order_wooshpay',
					this.onSubmit
				);

			// pay order page
			if ( $( 'form#order_review' ).length ) {
				this.form = $( 'form#order_review' );
			}

			$( 'form#order_review, form#add_payment_method' )
				.on(
					'submit',
					this.onSubmit
				);

			$( 'form.woocommerce-checkout' )
				.on(
					'change',
					this.reset
				);

			// add payment method page
			if ( $( 'form#add_payment_method' ).length ) {
				this.form = $( 'form#add_payment_method' );
			}

			$( document )
				.on(
					'wooshpayError',
					this.onError
				)
				.on(
					'checkout_error',
					this.reset
				);
		},

		/**
		 * Check to see if wooshpay in general is being used for checkout.
		 *
		 * @return {boolean}
		 */
		isWooshpayChosen: function() {
			return $( '#payment_method_wooshpay' ).is( ':checked' );
		},

		/**
		 * Checks if a payment method id is present as a hidden input.
		 * @return {boolean}
		 */
		hasSource: function() {
			return 0 < $('[name="wooshpay_pm_id"]').length;
		},

		/**
		 * Check whether a mobile device is being used.
		 *
		 * @return {boolean}
		 */
		isMobile: function() {
			if( /Android|webOS|iPhone|iPad|iPod|BlackBerry|IEMobile|Opera Mini/i.test( navigator.userAgent ) ) {
				return true;
			}

			return false;
		},

		/**
		 * Blocks payment forms with an overlay while being submitted.
		 */
		block: function() {
			if ( ! wc_wooshpay_form.isMobile() ) {
				wc_wooshpay_form.form.block( {
					message: null,
					overlayCSS: {
						background: '#fff',
						opacity: 0.6
					}
				} );
			}
		},

		/**
		 * Removes overlays from payment forms.
		 */
		unblock: function() {
			wc_wooshpay_form.form && wc_wooshpay_form.form.unblock();
		},

		/**
		 * Returns the selected payment method HTML element.
		 *
		 * @return {HTMLElement}
		 */
		getSelectedPaymentElement: function() {
			return $( '.payment_methods input[name="payment_method"]:checked' );
		},

		/**
		 * Retrieves "owner" data from either the billing fields in a form or preset settings.
		 *
		 * @return {Object}
		 */
		getOwnerDetails: function() {
			var first_name = $( '#billing_first_name' ).val(),
				last_name  = $( '#billing_last_name' ).val(),
				owner      = { name: '', address: {}, email: '', phone: '' };

			owner.name = first_name;

			if ( first_name && last_name ) {
				owner.name = first_name + ' ' + last_name;
			}

			owner.email = $( '#billing_email' ).val();
			owner.phone = $( '#billing_phone' ).val();

			if ( typeof owner.phone === 'undefined' || 0 >= owner.phone.length ) {
				delete owner.phone;
			}

			if ( typeof owner.email === 'undefined' || 0 >= owner.email.length ) {
				delete owner.email;
			}

			if ( typeof owner.name === 'undefined' || 0 >= owner.name.length ) {
				delete owner.name;
			}

			owner.address.line1       = $( '#billing_address_1' ).val();
			owner.address.line2       = $( '#billing_address_2' ).val();
			owner.address.state       = $( '#billing_state' ).val();
			owner.address.city        = $( '#billing_city' ).val();
			owner.address.postal_code = $( '#billing_postcode' ).val();
			owner.address.country     = $( '#billing_country' ).val();

			return owner;
		},

		/**
		 * Initiates the creation of a payment method object.
		 */
		createSource: function() {
			var extra_details = wc_wooshpay_form.getOwnerDetails();
			var response = onConfirmMethod(this.event, extra_details);

			// FIXME 处理错误
			response.then( wc_wooshpay_form.sourceResponse );
		},

		sourceResponse: function( response ) {
			if (!response) {
				wc_wooshpay_form.form.unblock();
				return;
			}

			if (!response || !response.id) {
				wc_wooshpay_form.submitError('Error submitting to payment gateway, please try again');
				return;
			}

			wc_wooshpay_form.form.append(
				$( '<input type="hidden" />' )
					.attr( 'name', 'wooshpay_pm_id' )
					.val( response.id )
			);

			wc_wooshpay_form.form.trigger( 'submit' );
		},

		/**
		 * Performs payment-related actions when a checkout/payment form is being submitted.
		 *
		 * @return {boolean} An indicator whether the submission should proceed.
		 *                   WooCommerce's checkout.js stops only on `false`, so this needs to be explicit.
		 */
		onSubmit: function(e) {
			if ( ! wc_wooshpay_form.isWooshpayChosen() ) {
				return true;
			}
			if ( wc_wooshpay_form.hasSource() ) {
				return true;
			}

			wc_wooshpay_form.block();
			wc_wooshpay_form.createSource();
			return false;
		},

		/**
		 * Removes all errors and hidden fields with IDs from the form.
		 */
		reset: function() {
			$('[name="wooshpay_pm_id"]').remove();
			$( '.wc-wooshpay-error, .wooshpay-source' ).remove();
		},

		/**
		 * 
		 * @param {Event}  e      The jQuery event.
		 * @param {Object} result The result
		 */
		onError: function( e, result ) {
			var message = result.error.message;
			var selectedMethodElement = wc_wooshpay_form.getSelectedPaymentElement().closest( 'li' );
			var errorContainer;

			errorContainer = selectedMethodElement.find( '.wooshpay-source-errors' );

			// Notify users that the email is invalid.
			if ( 'email_invalid' === result.error.code ) {
				message = wc_wooshpay_params.email_invalid;
			} else if (
				/*
				 * Customers do not need to know the specifics of the below type of errors
				 * therefore return a generic localizable error message.
				 */
				'invalid_request_error' === result.error.type ||
				'api_connection_error'  === result.error.type ||
				'api_error'             === result.error.type ||
				'authentication_error'  === result.error.type ||
				'rate_limit_error'      === result.error.type
			) {
				message = wc_wooshpay_params.invalid_request_error;
			}

			if ( wc_wooshpay_params.hasOwnProperty(result.error.code) ) {
				message = wc_wooshpay_params[ result.error.code ];
			}

			wc_wooshpay_form.reset();
			$( '.woocommerce-NoticeGroup-checkout' ).remove();
			console.log( result.error.message ); // Leave for troubleshooting.
			$( errorContainer ).html( '<ul class="woocommerce_error woocommerce-error wc-wooshpay-error"><li /></ul>' );
			$( errorContainer ).find( 'li' ).text( message ); // Prevent XSS

			if ( $( '.wc-wooshpay-error' ).length ) {
				$( 'html, body' ).animate({
					scrollTop: ( $( '.wc-wooshpay-error' ).offset().top - 200 )
				}, 200 );
			}
			wc_wooshpay_form.unblock();
			$.unblockUI(); // If arriving via Payment Request Button.
		},

		/**
		 * Displays an error message in the beginning of the form and scrolls to it.
		 *
		 * @param {Object} error_message An error message jQuery object.
		 */
		submitError: function( error_message, is_html = false ) {
			if( ! is_html ) {
				var error = $( '<div><ul class="woocommerce-error"><li /></ul></div>' );
				error.find( 'li' ).text( error_message ); // Prevent XSS
				error_message = error.html();
			}

			$( '.woocommerce-NoticeGroup-checkout, .woocommerce-error, .woocommerce-message' ).remove();
			wc_wooshpay_form.form.prepend( '<div class="woocommerce-NoticeGroup woocommerce-NoticeGroup-checkout">' + error_message + '</div>' );
			wc_wooshpay_form.form.removeClass( 'processing' ).unblock();
			wc_wooshpay_form.form.find( '.input-text, select, input:checkbox' ).trigger( 'blur' );

			var selector = '';

			if ( $( '#add_payment_method' ).length ) {
				selector = $( '#add_payment_method' );
			}

			if ( $( '#order_review' ).length ) {
				selector = $( '#order_review' );
			}

			if ( $( 'form.checkout' ).length ) {
				selector = $( 'form.checkout' );
			}

			if ( selector.length ) {
				$( 'html, body' ).animate({
					scrollTop: ( selector.offset().top - 100 )
				}, 500 );
			}

			$( document.body ).trigger( 'checkout_error' );
			wc_wooshpay_form.unblock();
		}

	};

	wc_wooshpay_form.init();
} );
